from gm_types.ascle import ERROR
from helios.rpc.exceptions import RPCFaultException

from utils.base import APIView
from utils.channel_package_tools import ChannelPackage
from utils.logger import error_logger


class Vendor(APIView):
    def get(self, request):
        """
        获取渠道商列表
        :param request:
        :return:
        """
        query = request.GET.get('q', '')
        page = int(request.GET.get('page', 1))
        limit = int(request.GET.get('limit', 30))

        data = request.rpc.origin['endpoint'](
            query=query, page=page, limit=limit,
        ).unwrap()
        return data


class ChannelList(APIView):
    def get(self, request):
        """
        渠道列表
        :param request:
        :return:
        """
        page = int(request.GET.get('page', 1))
        limit = int(request.GET.get('limit', 10))
        filters = self.handle_filter(request.GET.get('filter', ""))
        try:
            channel_list = self.rpc['endpoint'](offset=(page - 1) * limit, limit=limit, filters=filters).unwrap()
        except Exception as e:
            error_logger.error(u'获取渠道列表失败：%s', e)
            raise e
        data = [
            {
                'name': data.get('name'),
                'url_name': data.get('url_name'),
                'pack': data.get('pack'),
                'download_url': data.get(),  # TODO 拼接下载链接 最近的一次发布
            }
            for data in channel_list]

        return self.write_success(message='获取渠道列表成功', data=data)


class Channel(APIView):
    def get(self, request):
        """
        渠道详情
        :param request:
        :return:
        """
        channel_id = request.GET.get('channel_id', 0)
        if not channel_id:
            error_logger.error(u'参数不完整')
            return self.write_fail(code=ERROR.ARG_MISS, message='参数不完整')
        try:
            channel_data = self.rpc['endpoint'](channel_id=channel_id).unwrap()
        except Exception as e:
            error_logger.error(u'获取渠道详情失败：%s', e)
            raise e
        channel = {
            'channel_id': channel_data.get('channel_id', 0),
            'channel_name': channel_data.get('name'),
            'url_name': channel_data.get('url_name'),
            'auto_pack': channel_data.get('pack'),
            'channel_vendor': channel_data.get('vendor'),
            'channel_type': channel_data.get('type'),
        }
        return self.write_success(message='获取渠道详情成功', data=channel)

    def post(self, request):
        """
        渠道创建 / 编辑
        :param request:
        :return:
        """
        channel_id = request.POST.get('channel_id', '')
        channel_name = request.POST.get('name', '')
        url_name = request.POST.get('url_name', '')
        auto_pack = request.POST.get('pack', True)
        channel_vendor = request.POST.get('vendor', 0)
        channel_type = request.POST.get('type', 2)
        if not all([channel_name, url_name, channel_vendor]):
            error_logger.error(u'参数不完整')
            return self.write_fail(code=ERROR.ARG_MISS, message='参数不完整')
        if channel_id:   # 编辑渠道信息
            try:
                self.rpc['endpoint'](
                    channel_id=channel_id, name=channel_name, pack=auto_pack,
                    vendor=channel_vendor, type=channel_type
                ).unwrap()
            except RPCFaultException as e:
                raise e
        else:  # 创建渠道信息
            try:
                self.rpc['endpoint'](
                    name=channel_name, pack=auto_pack, vendor=channel_vendor, type=channel_type
                ).unwrap()
            except RPCFaultException as e:
                raise e


class ChannelPacked(APIView):
    def get(self, request):
        """
        当前渠道已经打包的所有版本
        :param request:
        :return:
        """
        channel_id = request.GET.get('channel_id')
        if not channel_id:
            error_logger.error(u'参数不完整')
            return self.write_fail(code=ERROR.ARG_MISS, message='参数不完整')
        try:
            channel_data = self.rpc['endpoint'](channel_id=channel_id).unwrap()
        except Exception as e:
            error_logger.error(u'获取渠道打包列表失败：%s', e)
            raise e
        channel = [
            {
                'version': data.get('version', 0),
                'channel_name': data.get('name'),
                'packed_time': data.get('packed_time'),
                'release_time': data.get('release_time'),
                'is_released': data.get('is_released'),
                'download_url': '',  # TODO 拼接下载链接
            }
            for data in channel_data]
        return self.write_success(message='获取渠道打包列表成功', data=channel)

    def post(self, request):
        """
        当前版本的渠道包发布
        :param request:
        :return:
        """
        version2channel_id = request.POST.get('id')
        if not version2channel_id:
            error_logger.error(u'参数不完整')
            return self.write_fail(code=ERROR.ARG_MISS, message='参数不完整')

        version2channel = self.rpc['endpoint'](id=version2channel_id).unwrap()  # get
        version = version2channel['version']
        channel = version2channel['channel']
        ChannelPackage.publish_apk(channel, version)
        try:
            self.rpc['endpoint'](id=version2channel_id).unwrap()  # edit
        except RPCFaultException as e:
            error_logger.error(u'获取渠道打包列表失败：%s', e)
            return self.write_fail(code=-1, message='发布失败！')
        return self.write_success(data={}, message='发布成功！')
